#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <syslog.h>
#include <errno.h>
#ifdef sun
#include <sys/fcntl.h>
#endif
#include "popd.h"
#include "fdlist.h"
#include "userinfo.h"
#include "cmdlist.h"

char *sstrdup();
struct userlist *add_username();

struct fdlist *get_fdlist_entry();


/*
  Initialize fd userdata area and print out initial login message.
  This is called by the server module right after the socket is
  established.
*/

initial_message(d)
     int d;
{
  struct fdlist *fd;
  int i;
  extern int debug;

  char *msg="\"BOP 'til ya drop\" mailer version 0.9alpha";

  add_descriptor_timer(d);
  {
    int on=1;

    setsockopt(d,SOL_SOCKET,SO_KEEPALIVE,(char *)&on,sizeof(on));
    if(debug) printf("keepalive was %d\n",on&255);
  }
  i=fcntl(d,F_GETFL,0);
  if(fcntl(d,F_SETFL,i|O_NDELAY)) if(debug) perror("O_NDELAY:");
  fd=get_fdlist_entry(d);
  if(fd->userdata!=NULL) {
    syslog(LOG_ERR,"Userdata area not null for descriptor %d?\n",d);
    fd->userdata=NULL;
  }
  if(fd->userdata==NULL) {
    struct userdata *u;

    u=tmalloc(struct userdata);
    u->username=NULL;
    u->password=NULL;
    u->spoolfile=NULL;
    u->dropfile=NULL;
    u->userptr=NULL;
    u->writebuf=NULL;
    u->lastmessage=0;
    u->info.messlist=NULL;
    u->info.numdeleted=0;
    u->info.deletedsize=0;
    u->closeflag=0;
    u->filename=NULL;
    u->filedes=0-1;
    u->buf[0]=0;
    u->rdlen=0;
    u->beenmodified=0;
    u->writebufamt=0;
    u->desc_to_lock=0-1;
    u->desc_locked=0;
    fd->userdata=u;
  }
  write_ok(d,msg);
}

/* Reads data from the connection stream, and acts on it. */

read_request(socket,fd,abrt)
     int socket;
     struct fdlist *fd;
     int abrt;
{
  static char *cmdlist[50];
  char *cptr;
  int keyword,len,res;
  extern int debug;

  if(abrt) {
    abort_session(fd);
    return(0);
  }
  cptr=fd->userdata->buf;
  if((len=read(socket,cptr+fd->userdata->rdlen,512-fd->userdata->rdlen))<=0) {
    if(len<0 && errno==EWOULDBLOCK) return(0);
    else {
      abort_session(fd);
      return(1);
    }
  }
  touch_timer(socket);
  len+=fd->userdata->rdlen;
  cptr[len]=0;
  if(debug) {
    fprintf(stderr,"'%s'",cptr+fd->userdata->rdlen);
    fflush(stderr);
    if(strncmp(cptr,"exit",4)==0) exit(0);
  }
  if(fd->state!=8) {
    if(cptr[len-1]!='\n') {
      if(len==512) {
	/* For some reason the client decided to send us a ton of gibberish.
	   So we disconnect it. */
	abort_session(fd);
	return(1);
      }
      fd->userdata->rdlen=len;
      return(0);
    }
  }
  if(fd->state==8) {
    int x;

    if(cptr[len-3]=='.' && (len<4 || cptr[len-4]=='\n') && cptr[len-1]=='\n') {
      if(len>3) write(fd->userdata->filedes,cptr,len-3);
      fd->userdata->rdlen=0;
      finish_sending_message(socket,fd);
      return(0);
    }
    if(len>256) {
      len-=4;
      if(write(fd->userdata->filedes,cptr,len)!=len) {
	fd->userdata->rdlen=0;
	if(debug) fprintf(stderr,"Write to sendmail failed\n");
	abort_session(fd);
	return(1);
      }
      for(x=0;x<4;x++) cptr[x]=cptr[x+len];
      fd->userdata->rdlen=4;
    }
    else fd->userdata->rdlen=len;
    return(0);
  }
  fd->userdata->rdlen=0;
  if(parse_cmd(cptr,cmdlist,50,&keyword,socket)) return(0);
  if(keyword==KEY_NOOP) {
    write_ok(socket,"");
    return(0);
  }
  if(fd->state<2) {
    /* User hasn't logged in yet. */
    if(keyword==KEY_QUIT) return(1);
    if(fd->state==0) {
      if(keyword!=KEY_USER) {
	write_err(socket,"Unknown command: \"%s\".",cmdlist[0]);
	return(0);
      }
      fd->state=1;
      fd->userdata->username=sstrdup(cmdlist[1]);
      write_ok(socket,"Password required for %s.",cmdlist[1]);
      return(0);
    }
    else if(fd->state==1) {
      if(keyword!=KEY_PASS
#ifdef USE_RPOP
	 && keyword!=KEY_RPOP
#endif
	 ) {
	write_err(socket,"Unknown command: \"%s\"",cmdlist[0]);
	return(0);
      }
      if((res=getuserinfo(fd->userdata->username,cmdlist[1],&(fd->userdata->spoolfile),keyword==KEY_RPOP,socket)) || (fd->userdata->userptr=add_username(fd->userdata->username))==NULL) {
	if(res==0) {
	  write_err(socket,"Session for \"%s\" already exists.",fd->userdata->username);
	}
	else {
	  syslog(LOG_INFO,"User %s failed login attempt from %s",fd->userdata->username,gethostforsocket(&(fd->sock_addr)));
	  write_err(socket,"Password supplied for \"%s\" is incorrect.",fd->userdata->username);
	}
	sfree(fd->userdata->username);
	fd->userdata->username=NULL;
	if(fd->userdata->spoolfile!=NULL) {
	  sfree(fd->userdata->spoolfile);
	  fd->userdata->spoolfile=NULL;
	}
	fd->state=0;
	return(0);
      }
      update_login_time(fd->userdata->username,"popmail",gethostforsocket(&(fd->sock_addr)),time(0));
      fd->state=2;
      count_and_parse_messages(socket,fd);
      return(0);
    }
    return(0);
  }
  switch(keyword) {
  case KEY_QUIT:
    write_out_messages(socket,fd);
    break;
  case KEY_RETR:
    send_message(socket,fd,atoi(cmdlist[1]),-1);
    break;
  case KEY_DELE:
    fd->userdata->beenmodified=1;
    delete_message(socket,fd,atoi(cmdlist[1]));
    break;
  case KEY_RSET:
    undelete_messages(socket,fd,-1);
    fd->userdata->lastmessage=1;
    break;
  case KEY_STAT:
    pop_stat(socket,fd);
    break;
  case KEY_LIST:
    if(cmdlist[1]!=NULL) pop_list(socket,fd,atoi(cmdlist[1]));
    else pop_list(socket,fd,-1);
    break;
  case KEY_TOP:
    send_message(socket,fd,atoi(cmdlist[1]),atoi(cmdlist[2]));
    break;
  case KEY_LAST:
    write_ok(socket,"%d",fd->userdata->lastmessage);
    break;
  case KEY_XTND:
    if(cmdlist[1]==NULL) {
      write_err(socket,"Missing argument to xtnd command");
      break;
    }
    if(strcmpi("XMIT",cmdlist[1])==0) pop_xmit(socket,fd,0);
    else {
      write_err(socket,"Unknown extended command \"%s\"",cmdlist[1]);
    }
    break;
  }
  return(0);
}

/* Abort session without saving changes to the mail drop. This will
   leave a .drop file around if the spool file is currently locked. We
   can't set an alarm to do locking because the descriptor may have been
   closed at this point, so we get one shot and then byebye. */

abort_session(fd)
     struct fdlist *fd;
{
  unlock_spool_file(fd);
}

free_userdata(fd)
     struct fdlist *fd;
{
  if(fd->userdata!=NULL) {
    if(fd->userdata->userptr!=NULL) {
      delete_username(fd->userdata->userptr);
      fd->userdata->userptr=NULL;
    }
    if(fd->userdata->spoolfile!=NULL) sfree(fd->userdata->spoolfile);
    if(fd->userdata->dropfile!=NULL) sfree(fd->userdata->dropfile);
    if(fd->userdata->username!=NULL) {
      syslog(LOG_INFO,"User %s logged out",fd->userdata->username);
      sfree(fd->userdata->username);
    }
    if(fd->userdata->writebuf!=NULL) free_writebufs(&(fd->userdata->writebuf));
    if(fd->userdata->filedes>0) close(fd->userdata->filedes);
    if(fd->userdata->filename!=NULL) {
      unlink(fd->userdata->filename);
      sfree(fd->userdata->filename);
    }
    do_free_messinfo(&(fd->userdata->info));
    free(fd->userdata);
    fd->userdata=NULL;
  }
}

/* This is the last function called right before the session socket is
   closed, so it's OK to free the data. */

remove_client_des(d)
     int d;
{
  struct fdlist *fd;

  turn_off_idletimer(d);
  fd=get_fdlist_entry(d);
  free_userdata(fd);
  remove_alarms_for_des(d);
}
