#include <stdio.h>
#include <signal.h>
#include <syslog.h>
#include "popd.h"
#include "fdlist.h"

static struct alarm {
  int time;
  PFI fun;
  int d;
  int fdesent;
  struct alarm *next;
} *headlist=NULL,*taillist=NULL;
static void do_set_alarm();
extern int set_alarm_flag();

/* Adds an alarm at time offset OFFSET. When the alarm expires, it calls
   function FUN with arguments D and FDES. */

add_alarm(offset,fun,d,fdes)
     int offset;
     PFI fun;
     int d;
     struct fdlist *fdes;
{
  struct alarm *a;
  int setalarm=0;
  int alarmt;

  alarmt=offset+time(0);
  a=tmalloc(struct alarm);
  a->time=alarmt;
  a->fun=fun;
  a->d=d;
  if(fdes!=NULL) a->fdesent=fdes->des;
  else a->fdesent=0-1;
  a->next=NULL;
  if(headlist==NULL) {
    headlist=a;
    taillist=a;
    setalarm=1;
  }
  else if(headlist->time>=alarmt) {
    a->next=headlist;
    headlist=a;
    setalarm=1;
  }
  else if(taillist->time<=alarmt) {
    taillist->next=a;
    taillist=a;
  }
  else {
    struct alarm *x,*prev;

    for(x=headlist;x->time<=alarmt;(prev=x),(x=x->next));
    prev->next=a;
    a->next=x;
  }
  if(setalarm) do_set_alarm(time(0));
}

/* Delete all alarms associated with descriptor d. */

remove_alarms_for_des(d)
     int d;
{
  struct alarm *ptr,*prev;
  int mask;

  prev=NULL;
  ptr=headlist;
  while(ptr!=NULL) {
    struct alarm *ptr2;

    if(ptr->d==d) {
      syslog(LOG_ERR,"Descriptor was deleted, but it has a pending alarm");
      if(prev==NULL) {
	headlist=headlist->next;
	if(headlist==NULL) taillist=NULL;
      }
      else {
	if(ptr->next==NULL) taillist=prev;
	prev->next=ptr->next;
      }
      ptr2=ptr;
      ptr=ptr->next;
      free(ptr2);
    }
    else {
      prev=ptr;
      ptr=ptr->next;
    }
  }
  do_set_alarm(time(0));
}

/* Alarm handler, runs all alarms that have gone off; can't be called
   directly from signal(). Must be called at a nice, clean, opportune
   time. */

do_alarm() {
  int rval=1;
  int currtime;

  if(headlist==NULL) return(1);
  currtime=time(0);
  while((headlist!=NULL) && (headlist->time<=currtime)) {
    struct alarm *b;

    if(headlist->fdesent>=0) {
      (headlist->fun)(headlist->d,get_fdlist_entry(headlist->fdesent));
    }
    else {
      (headlist->fun)(headlist->d,NULL);
    }
    b=headlist->next;
    free(headlist);
    headlist=b;
    if(headlist==NULL) taillist=NULL;
    rval=0;
  }
  do_set_alarm(currtime);
  return(rval);
}

int very_late_alarm() {
  if(headlist==NULL) return(0);
  return((time(0)-headlist->time)>5);
}

static void do_set_alarm(currtime)
     int currtime;
{
  int offset;

  if(headlist==NULL) {
    alarm(0);
    return;
  }
  offset=headlist->time-currtime;
  if(offset<=0) set_alarm_flag();
  else alarm(offset);
}
