#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <errno.h>
#ifdef sun
#include <sys/fcntl.h>
#endif
#include "popd.h"
#include "userinfo.h"
#include "fdlist.h"

struct fdlist *get_fdlist_entry();
int read_some_messages();
int try_lock_for_count();
extern char *sstrdup();
static char romline[]="rom ";
static char readonlyline[]=" RO\n";
static char statusline[]="Status:";

/* OK, so how does this work? Well, it first tries to lock the file. If
   this succeeds, it renames it to file.lock and then counts the messages.
   Otherwise, it schedules an alarm to repeatedly invoke try_lock_for_count
   until it manages to lock the file.

   During this time the socket associated with the session is locked. This
   is so that an event will not disrupt the access of the file. This also
   has the unfortunate side effect of ignoring terminated sessions. */

count_and_parse_messages(socket,fdes)
     int socket;
     struct fdlist *fdes;
{
  struct stat s;
  char drop[512];

  if(fdes->userdata==NULL) return(1);
  if(fdes->userdata->dropfile==NULL) {
    strcpy(drop,fdes->userdata->spoolfile);
    strcat(drop,".drop");
    fdes->userdata->dropfile=sstrdup(drop);
    fdes->userdata->info.nummessages=0;
  }
  if(lstat(fdes->userdata->dropfile,&s)==0) {
    /* There is a dropfile there, let's check it out */
    if(s.st_mode&S_IFREG) {
      if(unlock_spool_file(fdes)) {
	add_alarm(5,count_and_parse_messages,socket,fdes);
	return(1);
      }
    }
    unlink(fdes->userdata->dropfile);
  }
  /* User has no spool file. */
  if(!fileok(fdes->userdata->spoolfile,-1,NULL)) {
    unlink(fdes->userdata->spoolfile); /* Blow it away, in case it was silly */
    fdes->userdata->info.filesize=0;
    return(return_from_count(socket,fdes));
  }
  lockdes(socket);
  if(lock_spool_file(fdes)) {
    add_alarm(5,try_lock_for_count,socket,fdes);
    return(1);
  }
  else return(finish_count(socket,fdes));
}

/* Try to get the lock for the file. */

try_lock_for_count(socket,fdes)
     int socket;
     struct fdlist *fdes;
{
  extern int debug;

  if(debug) printf("Trying lock\n");
  if(fdes->userdata==NULL) return(0);
  if(lock_spool_file(fdes)) {
    add_alarm(5,try_lock_for_count,socket,fdes);
    return(1);
  }
  else return(finish_count(socket,fdes));
}

/* Make sure the file is OK, and if it is, count how many messages are
   in it, find all the Status: lines (or note if there are missing ones),
   etc. 

   The actual count is done by a thread that is invoked whenever data is
   available to be read on the open file descriptor.
*/

finish_count(socket,fdes)
     int socket;
     struct fdlist *fdes;
{
  FILE *fd;
  struct fdlist *d;
  struct stat s,s2;
  int fn;

  if(fdes->userdata==NULL) return(-2);
  if(lstat(fdes->userdata->dropfile,&s)==0) {
    if(!(s.st_mode&S_IFREG)) {
      /* Kidz are kidz...and they'll never learn */
      unlink(fdes->userdata->dropfile);
      unlock_spool_file(fdes);
      unlockdes(socket);
      return(-2);
    }
  }
#ifdef USE_STDIO
  if((fd=fopen(fdes->userdata->dropfile,"r"))==NULL) {
    unlock_spool_file(fdes);
    unlockdes(socket);
    return(-2);
  }
  fn=fileno(fd);
#else
  fd=NULL;
  if((fn=open(fdes->userdata->dropfile,O_RDONLY|O_NDELAY))<0) {
    unlock_spool_file(fdes);
    unlockdes(socket);
    return(-2);
  }
#endif
  if(fstat(fn,&s2) || (s2.st_ino!=s.st_ino) || (s.st_dev!=s2.st_dev)) {
    close(fn);
    if(fd!=NULL) fclose(fd);
    unlock_spool_file(fdes);
    unlockdes(socket);
    return(-2);
  }
  fdes->userdata->got_status=0;
  add_des(fn,13,read_some_messages,NULL,fd);
  d=get_fdlist_entry(fn);
  d->reqnum=socket;
#ifdef USE_STDIO
  d->state=0;
#else
  d->state=8;
#endif
  d->blksz=0;
  d->send_group_des=0;
  d->send_pass_des=0;
  return(0);
}

#ifdef USE_STDIO
read_some_messages(socket,fd,abrt)
     int socket;
     struct fdlist *fd;
     int abrt;
{
  char buf[512];
  int count=0,currpos;
  struct fdlist *d;
  char *c;

  if(abrt) {
    unlockdes(fd->reqnum);
    return(0);
  }
  d=get_fdlist_entry(fd->reqnum);
  if(d==NULL) return(1);
  if(d->userdata==NULL) return(1);
  touch_timer(fd->reqnum); /* We'll count this as not being idle */
  while((count < 100) && ((c=fgets(buf,512,fd->fd))!=NULL)) {
    if((buf[0]=='F') && (strncmp(buf,"From ",5)==0)) {
      fd->state=0;
      currpos=ftell(fd->fd)-strlen(buf);
      add_message(d,currpos);
    }
    else if(fd->state==0) {
      if(buf[0]=='\n') {
	fd->state=1;
	set_header_end(d,ftell(fd->fd)-1);
	if(d->userdata->got_status==0) {
	  extern int debug;

	  d->userdata->beenmodified=1;
	  if(debug) printf("Beenmodified set in read_some_message: message %d\n",d->userdata->info.nummessages);
	}
	else d->userdata->got_status=0;
      }
      else {
	if((buf[0]=='S') && (buf[6]==':') && (strncmp(buf,"Status:",7)==0)) {
	  int len,f;

	  d->userdata->got_status=1;
	  len=strlen(buf);
	  currpos=ftell(fd->fd)-len;
	  if((buf[8]=='R') && (strncmp(buf+7," RO\n",4)==0)) f=1;
	  else {
	    extern int debug;

	    if(debug) printf("Message %d doesn't have status: RO set\n",d->userdata->info.nummessages);
	    f=0;
	  }
	  set_status_pos(d,currpos,len,f);
	}
      }
    }
    count++;
  }
  if(c==NULL) {
    struct stat s;

    if(fstat(fileno(fd->fd),&s)==0) {
      int n;

      n=d->userdata->info.nummessages-1;
      d->userdata->info.filesize=s.st_size;
      if(n>=0) {
	d->userdata->info.messlist[n].length=s.st_size-d->userdata->info.messlist[n].offset;
      }
    }
    fclose(fd->fd);
    return_from_count(fd->reqnum,d);
    unlockdes(fd->reqnum);
    return(1);
  }
  return(0);
}

#else

/* This is a scary piece of code. It doesn't use stdio; instead, it processes
   the input file READAMT bytes at a time, using a nasty state machine to parse
   the file. */
#define READAMT 2048

read_some_messages(socket,fd,abrt)
     int socket;
     struct fdlist *fd;
     int abrt;
{
  char buf[READAMT]; /* Holds the data read from the file */
  int count; /* number of blocks read so far */
  int currpos; /* offset within file */
  struct fdlist *d; /* descriptor pointer */
  register char c; /* temp char */
  register char *bptr; /* Pointer within buf */
  register char *bend; /* end of data in buf */
  int amt; /* amount read from file */

  if(abrt) {
    unlockdes(fd->reqnum);
    return(0);
  }
  d=get_fdlist_entry(fd->reqnum);
  if(d==NULL || d->userdata==NULL) return(1);
  touch_timer(fd->reqnum); /* We'll count this as not being idle */
  for(count=0;count<8;count++) {
    amt=read(socket,buf,READAMT);
    if(amt<0 && errno==EWOULDBLOCK) return(0);
    currpos=fd->send_pass_des;
    fd->send_pass_des+=amt;
    if(amt<=0) {
      /* All done, or there was an error. Either way, we quit. */
      struct stat s;

      if(fstat(socket,&s)==0) {
	int n;

	n=d->userdata->info.nummessages-1;
	d->userdata->info.filesize=s.st_size;
	if(n>=0) {
	  d->userdata->info.messlist[n].length=s.st_size-d->userdata->info.messlist[n].offset;
	}
      }
      return_from_count(fd->reqnum,d);
      unlockdes(fd->reqnum);
      return(1);
    }
    bptr=buf;
    bend=buf+amt;
    while(bptr<bend) { /* While there's still data left to be processed */
      switch(fd->state) {
      case 0:
	/* At the beginning of a line, in the header */
	c=(*(bptr++));
	if(c=='\n') {
	  extern int debug;

	  if(debug>1) printf("End of header reached, pos is %d\n",fd->send_alias_des);
	  /* End of header */
	  set_header_end(d,currpos+(bptr-buf)-1);
	  if(d->userdata->got_status==0) d->userdata->beenmodified=1;
	  else d->userdata->got_status=0;
	  fd->state=8;
	  goto state8;
	}
	if(c=='S') {
	  fd->send_alias_des=currpos+(bptr-buf)-1;
	  /* Might be a Status: line */
	  fd->blksz=0;
	  fd->state=2;
	  fd->send_group_des=1;
	  goto state2;
	}
	fd->state=4;
	goto state4;
	break;
      case 2:
      state2:
	/* Check to see if the rest of line is a Status: header */
	while(bptr<bend) {
	  if(statusline[fd->send_group_des++]!=(*(bptr++))) {
	    fd->state=4;
	    goto state4;
	  }
	  if(statusline[fd->send_group_des]==0) {
	    fd->state=3;
	    d->userdata->got_status=1;
	    fd->blksz=0;
	    goto state3;
	  }
	}
	break;
      case 3:
      state3:
	/* It's a status line, now look for " RO\n" */
	while(bptr<bend) {
	  c=(*(bptr++));
	  fd->send_group_des++;
	  if(readonlyline[fd->blksz++]!=c) {
	    if(c=='\n') {
	      set_status_pos(d,fd->send_alias_des,fd->send_group_des,0);
	      fd->state=0;
	      break;
	    }
	    fd->state=5;
	    goto state5;
	  }
	  if(readonlyline[fd->blksz]==0) {
	    extern int debug;

	    if(debug) {
	      if((currpos+(bptr-buf)-fd->send_alias_des)!=fd->send_group_des) {
		printf("Amounts differ %d %d...\n",(currpos+(bptr-buf)-fd->send_alias_des),fd->send_group_des);
	      }
	    }
	    set_status_pos(d,fd->send_alias_des,fd->send_group_des,1);
	    fd->state=0;
	    break;
	  }
	}
	break;
      case 4:
      state4:
	/* Skip to end of line, then goto state 0 */
	while(bptr<bend) {
	  if((*(bptr++))=='\n') {
	    fd->state=0;
	    break;
	  }
	}
	break;
      case 5:
      state5:
	/* Skip to end of line, then set status: position and length */
	while(bptr<bend) {
	  fd->send_group_des++;
	  if((*(bptr++))=='\n') {
	    set_status_pos(d,fd->send_alias_des,fd->send_group_des,0);
	    fd->state=0;
	    break;
	  }
	}
	break;
      case 8:
      state8:
	/* At start of line in body, looking for next "From " line */
	if(bptr<bend) {
	  c=(*(bptr++));
	  if(c=='\n') goto state8;/* Skip blank lines */
	  if(c=='F') {
	    fd->send_alias_des=currpos+(bptr-buf)-1;
	    /* Might be "From " line, let's find out */
	    fd->blksz=0;
	    fd->state=10;
	    goto state10;
	  }
	  /* Skip to end of line */
	  fd->state=9;
	  goto state9;
	}
	break;
      case 9:
      state9:
	/* Skip to end of line, then enter state 8 */
	while(bptr<bend) {
	  if((*(bptr++))=='\n') {
	    fd->state=8;
	    goto state8;
	  }
	}
	break;
      case 10:
      state10:
	/* Looking for rest of "From " line */
	while(bptr<bend) {
	  c=(*(bptr++));
	  if(romline[fd->blksz++]!=c) {
	    if(c=='\n') {
	      fd->state=8;
	      goto state8;
	    }
	    else {
	      fd->state=9;
	      goto state9;
	    }
	  }
	  if(romline[fd->blksz]==0) {
	    extern int debug;

	    if(debug>1) printf("Start of header at pos %d\n",fd->send_alias_des);
	    add_message(d,fd->send_alias_des);
	    d->userdata->got_status=0;
	    fd->state=4;
	    goto state4;
	  }
	}
	break;
      }
    }
  }
  return(0);
}

#endif

return_from_count(socket,fdes)
     int socket;
     struct fdlist *fdes;
{
  write_ok(socket,"%s has %d messages (%d octets).",fdes->userdata->username,fdes->userdata->info.nummessages,fdes->userdata->info.filesize);
  fdes->state=3; /* 3 = normal operation */
  return(0);
}

/* Add a message that starts at position pos in the file */

add_message(fdes,pos)
     struct fdlist *fdes;
     int pos;
{
  struct fileinfo *i;
  int n;

  i=(&(fdes->userdata->info));
  n=i->nummessages++;
  if(n==0) i->messlist=tmalloc(struct messlist);
  else {
    tnrealloc(i->messlist,struct messlist,i->nummessages);
    i->messlist[n-1].length=pos-i->messlist[n-1].offset;
  }
  i->messlist[n].flags=0;
  i->messlist[n].readflag=0;
  i->messlist[n].offset=pos;
  i->messlist[n].statuspos=0-1;
  i->messlist[n].statuslen=0-1;
  i->messlist[n].headerend=0-1;
}

/* Set end of header for current message */

set_header_end(d,end)
     struct fdlist *d;
     int end;
{
  int n;

  n=d->userdata->info.nummessages-1;
  d->userdata->info.messlist[n].headerend=end;
}

/* Set position of Status: line; len is the length of the status line, and
   readflag indicates if the message has been read or not */

set_status_pos(d,pos,len,readflag)
     struct fdlist *d;
     int pos,len,readflag;
{
  int n;

  n=d->userdata->info.nummessages-1;
  d->userdata->info.messlist[n].statuspos=pos;
  d->userdata->info.messlist[n].statuslen=len;
  d->userdata->info.messlist[n].readflag=readflag;
}

/* Free up the message information */

do_free_messinfo(p)
     struct fileinfo *p;
{
  if(p->messlist!=NULL) {
    free(p->messlist);
    p->messlist=NULL;
  }
}

fileok(name,size,s)
     char *name;
     int size;
     struct stat *s;
{
  struct stat sp;

  if(s==NULL) s=(&sp);
  if(lstat(name,s) || (!(s->st_mode&S_IFREG))) return(0);
  if(size>=0 && s->st_size!=size) return(0);
  return(1);
}
