#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <syslog.h>
#ifdef sun
#include <sys/fcntl.h>
#endif
#include "popd.h"

#include "fdlist.h"
#include "userinfo.h"

/* This is a misnomer. It actually moves the spool file to a .drop file. */

lock_spool_file(fd)
     struct fdlist *fd;
{
  char buf[512];
  extern int debug;

  if(fd==NULL || fd->userdata==NULL) return(0);
  if(fd->userdata->spoolfile==NULL) return(0);
  if(do_mail_lock(fd->userdata->spoolfile,buf)) return(1);
  if(link(fd->userdata->spoolfile,fd->userdata->dropfile)) {
    syslog(LOG_ERR,"Couldn't link %s to %s: %m\n",fd->userdata->spoolfile,fd->userdata->dropfile);
    if(debug) {
      printf("Couldn't link %s to %s\n",fd->userdata->spoolfile,fd->userdata->dropfile);
      fflush(stdout);
    }
    do_mail_unlock(buf);
    return(2);
  }
  unlink(fd->userdata->spoolfile);
  do_mail_unlock(buf);
  return(0);
}

/* This is a misnomer. It actually moves the .drop file back to the spool,
   appending any messages that have come in. Returns 0 on success, 1
   if the file was locked */

unlock_spool_file(fd)
     struct fdlist *fd;
{
  char buf[512],buf2[512];
  int i,r,len;
  struct stat s,s2;
  extern int debug;

  if(fd==NULL || fd->userdata==NULL) return(0);
  if(fd->userdata->spoolfile==NULL) return(0);
  if(fd->userdata->dropfile==NULL) return(0);
  if(lstat(fd->userdata->dropfile,&s)) return(0); /* No drop file */
  if(!(s.st_mode&S_IFREG)) { /* Someone's screwing around; blow it away */
    unlink(fd->userdata->dropfile); /* Bad if it's a directory. But kidz will
				       suffer the sins of their fathers. */
    return(0); /* There's no dropfile, so we're done */
  }
  lstat(fd->userdata->spoolfile,&s2);
  if(s2.st_ino==s.st_ino) {
    if(debug) printf("dropfile linked to spoolfile\n");
    unlink(fd->userdata->dropfile);
    return(0);
  }
  /* Trying to minimize the race condition possibilities. I don't know of
      any good solution to this one. */
  i=open(fd->userdata->dropfile,O_WRONLY|O_APPEND);
  if(i<0) return(0);
  if(fstat(i,&s2) || (s2.st_ino!=s.st_ino) || (s2.st_dev!=s.st_dev)) {
    close(i);
    return(0);
  }
  if(do_mail_lock(fd->userdata->spoolfile,buf)) {
    close(i);
    return(1);
  }
  if(lstat(fd->userdata->spoolfile,&s2)) r=0-1;
  else if(!(s2.st_mode&S_IFREG)) {
    unlink(fd->userdata->spoolfile);
    r=0-1;
  }
  else r=open(fd->userdata->spoolfile,O_RDONLY);
  if(r>=0) {
  /* This blocks the server; normally, however, this will be short [Famous
     last words.] We could go through the agony of setting up another event
     queued descriptor, and run into all sorts of nasty problems because we
     leave the spool file linked.*/

    if(fstat(r,&s) || (s.st_ino!=s2.st_ino) || (s.st_dev!=s2.st_dev)) {
      close(r);
    }
    else {
      while((len=read(r,buf2,512))>0) write(i,buf2,len);
      close(r);
      unlink(fd->userdata->spoolfile);
    }
  }
  close(i);
  rename(fd->userdata->dropfile,fd->userdata->spoolfile);
  do_mail_unlock(buf);
  return(0);
}

/* Lock the spool file with a .lock file. Attempts to link the spool
   file to the lock file; if the spool file isn't a regular file it
   deletes it and fails. If the spool file doesn't exist, it tries to
   creat() the lock file. 

   This function returns a 0 on success, and a 1 on failure (the lock
   already exists). It will be repeatedly called until the lock
   succeeds. */

do_mail_lock(name,buf)
     char *name;
     char *buf;
{
  struct stat s;
  extern int debug;

  strcpy(buf,name);
  strcat(buf,".lock");
  if(debug) fprintf(stderr,"Trying to link %s to %s\n",name,buf);
  if(lstat(name,&s)) {
    int d;

    /* There's no spool file, so we creat() the lock file */
    if(lstat(buf,&s)==0) return(1);
    if((d=creat(buf,0600))<0) return(1);
    close(d);
    return(0);
  }
  if(!(s.st_mode&S_IFREG)) {
    /* Sigh */
    unlink(name);
    return(1);
  }
  if(link(name,buf)!=0) return(1);
  return(0);
}

/* Unlock it. Simple, huh? Presumes we own the lock, among other obsenities. */

do_mail_unlock(buf)
     char *buf;
{
  unlink(buf);
}
