#ifdef ultrix
/* 4096? Come on. Copying fd_set structures suddenly becomes reasonable
   with only 64 descriptors defined. */
#define FD_SETSIZE 64
#endif
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#ifdef DGUX
#include <arpa/inet.h>
#endif
#include <netdb.h>
#if defined(UPDATEPASSWD) || defined(USE_BP) || defined(ISKERBEROS)
#include <krb.h>
#include "update_passwd.h"
#else
#include "mymalloc.h"
typedef int (*PFI)();
#endif
#include <syslog.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/file.h>
#include "fdlist.h"
#ifdef USE_DOMAINS
#include "domain.h"
#endif
#ifdef UPDATEPASSWD
#include "getservent.h"
#endif
struct userdata {
  PFI readfun,closefun,initialize;
};
static PFI notifyclosefun=NULL;


static int print_error(errnum,src)
     int errnum;
     char *src;
{
  if(errnum==0) fprintf(stderr,src);
  else {
    errno=errnum;
    perror(src);
  }
}

static PFI do_print_error=print_error;
static int freedesmem();
static int unauth_sock=0-1,unauth_sock_udp=0-1;
static char ourservice[512];
static char ourhostname[64];		/* for hostname */
static int accept_unauth(),accept_unauth_global();
char *process_command();

setourhostname(ouraddr,servicename)
     struct in_addr *ouraddr;
     char *servicename;
{
  struct hostent *host;

  if (gethostname(ourhostname, sizeof(ourhostname)) < 0) {
    do_print_error(errno,"gethostname");
    return;
  }

  if ((host = gethostbyname(ourhostname)) == (struct hostent *)0) {
    fprintf(stderr, "%s: host unknown\n", ourhostname);
    return;
  }
  bcopy(host->h_addr, (char *)ouraddr, host->h_length);
  if(servicename!=NULL) strcpy(ourservice,servicename);
}

char *getourhostname() {
  return(ourhostname);
}

/* Initializes a listener on service SERVICENAME (assumed to be
   a tcp service). Initialize will be invoked when a connection
   is made; readfun will be called when data is available to be
   read on the socket; and closefun will be invoked when the
   socket is closed. */

void init_unauth_global(servicename,readfun,closefun,initialize)
     char *servicename;
     PFI readfun,closefun,initialize;
{
  struct in_addr ouraddr;
  struct fdlist *fd;

  setourhostname(&ouraddr,NULL);
  unauth_sock=create_socket(ouraddr,servicename,"tcp",NULL);
  if(unauth_sock<0) {
    fprintf(stderr,"Creating server socket failed\n");
    exit(1);
  }
  add_des(unauth_sock,1,accept_unauth_global,NULL,NULL);
  fd=get_fdlist_entry(unauth_sock);
  fd->userdata=tmalloc(struct userdata);
  fd->userdata->readfun=readfun;
  fd->userdata->closefun=closefun;
  fd->userdata->initialize=initialize;
  set_close_fun(unauth_sock,freedesmem);
}

set_error_fun(errfun)
     PFI errfun;
{
  do_print_error=errfun;
}

set_notify_closefun(fun)
     PFI fun;
{
  notifyclosefun=fun;
}

shutdownunauthserver() {
  if(unauth_sock>0) {
    shutdown(unauth_sock,2);
    close_des(unauth_sock);
  }
  if(unauth_sock_udp>0) {
    shutdown(unauth_sock_udp,2);
    close_des(unauth_sock);
  }
}

init_unauth(servicename,readfun,closefun,initialize,udpreadfun)
     char *servicename;
     PFI readfun,closefun,initialize,udpreadfun;
{
  struct in_addr localaddr;
  struct fdlist *fd;

  localaddr.s_addr=htonl(0x7f000001); /*127.0.0.1*/
  unauth_sock=create_socket(localaddr,servicename,"tcp",NULL);
  if(unauth_sock<0) return;
  add_des(unauth_sock,1,accept_unauth,NULL,NULL);
  fd=get_fdlist_entry(unauth_sock);
  fd->userdata=tmalloc(struct userdata);
  set_close_fun(unauth_sock,freedesmem);
  fd->userdata->readfun=readfun;
  fd->userdata->closefun=closefun;
  fd->userdata->initialize=initialize;
  if(udpreadfun!=NULL) {
    unauth_sock_udp=create_socket(localaddr,servicename,"udp",NULL);
    if(unauth_sock_udp<0) {
      do_print_error(0,"Creating udp server socket failed\n");
      return;
    }
    add_des(unauth_sock_udp,1,udpreadfun,NULL,NULL);
    printf("%d is unauth_sock\n",unauth_sock_udp);
  }
}

/* Create a socket bound locally to localaddr listening on service
   SERVICENAME of type SERVICETYPE (SERVICETYPE can either be "tcp"
   or "udp"). If sptr != NULL, it will be filled in with the address
   that the socket is listening on. Either the socket will be returned,
   or a negative value in case of error */

create_socket(localaddr,servicename,servicetype,sptr)
     struct in_addr localaddr;
     char *servicename,*servicetype;
     struct sockaddr_in *sptr;
{
  int res;
  struct servent *serv;
  char *tp;
  int dgram_socket;
  struct sockaddr_in tmp_info;

  if(sptr==NULL) sptr=(&tmp_info);
  bzero((char *)sptr, sizeof(*sptr));
  sptr->sin_family = AF_INET;
  
  /* Look up service */
  if ((serv = getservbyname(servicename, servicetype)) == NULL) {
    char buf[512];

    sprintf(buf,"service unknown: %s/%s\n", servicename,servicetype);
    do_print_error(0,buf);
    return(-1);
  }
  sptr->sin_port = serv->s_port;
  sptr->sin_addr = localaddr;

  for(tp=ourhostname;*tp && *tp!='.';tp++);
  *tp=0;

  /* Open socket */
  if(strcmp(servicetype,"tcp")==0) dgram_socket=0;
  else dgram_socket=1;
try_again:
  if(dgram_socket==0) {
    if ((res = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      do_print_error(errno,"opening datagram socket");
      return(-1);
    }
    {
      int on=1;

      setsockopt(res,SOL_SOCKET,SO_REUSEADDR,(char *)&on,sizeof(on));
    }
  } else {
    if ((res = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      do_print_error(errno,"opening datagram socket");
      return(-1);
    }
  } 
  
  /* Bind the socket */
  if (bind(res, (struct sockaddr *)sptr, sizeof(*sptr))) {
    if(errno==EADDRINUSE) {
      if(connect(res,(struct sockaddr *)sptr,sizeof(*sptr))<0) {
	if(errno==ECONNREFUSED) {
	  do_print_error(0,"Connection not allowed-trying again\n");
	  close(res);
	  sleep(2);
	  goto try_again;
	}
      }
      {
	char buf[512];

	sprintf(buf,"There is already an active listener on service %s\n",servicename);
	do_print_error(0,buf);
      }
      close(res);
      return(-1);
    }
    {
      char buf[512];
      
      sprintf(buf,"creating socket for %s",servicename);
      do_print_error(errno,buf);
    }
    return(-1);
  }
  if(!dgram_socket) {
    if(listen(res,8)) do_print_error(errno,"Error in listen");
  }
  return(res);
}

static int accept_unauth_global(sock,ptr,abrt)
     int sock;
     struct fdlist *ptr;
     int abrt;
{
  int i;
  struct sockaddr_in c_sock;
  int datas;

  i=sizeof(c_sock);
  if((datas=accept(sock,&c_sock,&i))<0) {
    if(errno!=EINTR && errno!=EWOULDBLOCK && errno!=ENFILE && errno!=EMFILE) {
      do_print_error(errno,"accept");
      close(sock);
      return(1);
    }
    return(0);
  }
  add_des(datas,1,ptr->userdata->readfun,&c_sock,NULL);
  ptr=get_fdlist_entry(sock);
  if(ptr->userdata->closefun!=NULL) set_close_fun(datas,ptr->userdata->closefun);
  if(ptr->userdata->initialize!=NULL) ptr->userdata->initialize(datas);
  return(0);
}

static int freedesmem(sock)
     int sock;
{
  struct fdlist *fd;

  fd=get_fdlist_entry(sock);
  if(fd->userdata!=NULL) {
    free(fd->userdata);
    fd->userdata=NULL;
  }
  if(notifyclosefun!=NULL) notifyclosefun(sock);
  return(0);
}

static int accept_unauth(sock,ptr,abrt)
     int sock;
     struct fdlist *ptr;
     int abrt;
{
  int i;
  struct sockaddr_in c_sock;
  int datas;

  i=sizeof(c_sock);
  if((datas=accept(sock,&c_sock,&i))<0) {
    if(errno!=EINTR && errno!=EWOULDBLOCK) {
      do_print_error(errno,"accept");
      return(1);
    }
    return(0);
  }
  /* A bogus way of doing this, but it's portable */
  if(strcmp(inet_ntoa(c_sock.sin_addr),"127.0.0.1")) {
    close(datas);
    return(1);
  }
  add_des(datas,1,ptr->userdata->readfun,&c_sock,NULL);
  ptr=get_fdlist_entry(sock);
  if(ptr->userdata->closefun!=NULL) set_close_fun(datas,ptr->userdata->closefun);
  if(ptr->userdata->initialize!=NULL) ptr->userdata->initialize(datas);
  return(0);
}
