#include <stdio.h>
#include <sys/file.h>
#include <sys/wait.h>
#if defined(sun)
#include <sys/fcntl.h>
#endif
#include "popd.h"
#include "fdlist.h"
#include "userinfo.h"

#ifndef MAIL_COMMAND
#define MAIL_COMMAND "/usr/lib/sendmail"
#endif

#ifdef _AIX
#define vfork() fork()
#include <fcntl.h>
#ifndef SYSV_WAIT
#define SYSV_WAIT
#endif
#endif

#ifdef SOLARIS
#ifndef SYSV_WAIT
#define SYSV_WAIT
#endif
#endif

static struct proclist {
  int des,pid;
  struct proclist *next;
} *headlist=NULL;

char *sstrdup();
int check_child_death=0;

pop_xmit(socket,fd,num)
     int socket;
     struct fdlist *fd;
     int num;
{
  if(fd->state<2) return(1);
  fd->userdata->filename=sstrdup(tmpnam(NULL));
  if((fd->userdata->filedes=open(fd->userdata->filename,O_WRONLY|O_CREAT|O_TRUNC|O_EXCL,0600))<0) {
    write_err(socket,"Unable to open temp file.");
    sfree(fd->userdata->filename);
    fd->userdata->filename=NULL;
    return(1);
  }
  write_ok(socket,"Start sending the message.");
  fd->state=8;
  return(0);
}

finish_sending_message(socket,fd)
     int socket;
     struct fdlist *fd;
{
  int i;
  extern int debug;

  close(fd->userdata->filedes);
  fd->userdata->filedes=0-1;
  if(debug) fprintf(stderr,"Opening file %s, user %s\n",fd->userdata->filename,fd->userdata->username);
  i=vfork();
  if(i<0) {
    write_err(socket,"Unable to execute \"%s\"",MAIL_COMMAND);
    return(0);
  }
  if(i>0) {
    struct proclist *p;

    p=tmalloc(struct proclist);
    p->next=headlist;
    headlist=p;
    p->des=socket;
    p->pid=i;
    fd->state=6;
    return(0);
  }
  else {
    int d,i;

    i=getdtablesize();
    for(d=0;d<i;d++) close(d);
    if(open(fd->userdata->filename,O_RDONLY)<0) _exit(13);
    open("/dev/null",O_WRONLY);
    dup(1);
    execl(MAIL_COMMAND,"send-mail","-f",fd->userdata->username,"-t","-oem",NULL);
    _exit(14);
  }
}

child_death() {
  check_child_death++;
}

handle_child_death() {
#ifndef SYSV_WAIT
  union wait w;
#else
  int w;
#endif
  int pid;
  struct proclist *h,*prev;
  extern int debug;
  struct fdlist *fd;

  pid=wait(&w);
  if(debug) printf("Kid %d died\n",pid);
  h=headlist;
  prev=NULL;
  while(h!=NULL && h->pid!=pid) {
    prev=h;
    h=h->next;
  }
  if(h==NULL) return(1);
  if(prev!=NULL) prev->next=h->next;
  else headlist=h->next;
  fd=get_fdlist_entry(h->des);
  if(fd!=NULL) {
    if(fd->userdata!=NULL) {
      if(WEXITSTATUS(w)!=0) write_err(h->des,"Mailer exited with status %d",WEXITSTATUS(w));
      else write_ok(h->des,"Message sent successfully.");
      if(fd->userdata->filename!=NULL) {
	unlink(fd->userdata->filename);
	sfree(fd->userdata->filename);
	fd->userdata->filename=NULL;
      }
    }
    fd->state=2;
  }
  free(h);
  return(0);
}
