#! /local/bin/perl -w
#
# @(#) $Id: bozohttpd.pl,v 2.19 1995/11/04 12:40:20 mrg Exp $
#
# bozo httpd -- based on tiny httpd.
#
# currently supported: HTTP 1.0 GET
# file types of .html and .gif
#
# bozoleft(p) 1995 the bozotic softwar foundation
#
# author: mrg@mame.mu.oz.au

require 5;

$version = '3.0_BETA';

$0 =~ s#.*/##;

$config = '/etc/bozohttpd.cf';

$ENV{'PATH'} .= ":/usr/ucb";
require $config;
require 'bozocgi.pl' if $cgidir;
require 'bozobind.pl' if $tcpport;

death("no htmldir set in $config", 1) unless $htmldir;

$logfac = "auth" unless $logfac;
$logpri = "info" unless $logpri;

use Socket 'AF_INET';

# Messages
%errors =
(
	"403" => "Forbidden",
	"404" => "Not Found",
	"500" => "Internal Error",
	"501" => "Not Implemented",
);

%verrors =
(
	"403" => "Your client is not allowed to request this item",
	"404" => "The requested item was not found on this server",
	"500" => "An error occurred while trying to retrieve item",
	"501" => "This server does not support the given request type",
);

$ENV{'SERVER_SOFTWARE'} = "bozohttpd $version";
$sockaddr = 'S n a4 x8';

daemon() if $tcpport;		# daemon will fork/bg and bind to $tcpport
				# and call do_connection for us.
do_connection();

sub do_connection
{
	($addr = getpeername(STDIN)) || death("getsockaddr: $!");
	($inetaddr) = (unpack($sockaddr, $addr))[2];
	($host, $aliases) = gethostbyaddr($inetaddr, AF_INET);
	@inetaddr = unpack('C4', $inetaddr);
	$inetaddr = join('.', @inetaddr);
	@host = split(' ', "$host $aliases");
	$host = $inetaddr unless $host;
	$| = 1;

	logit("connect from $host") if $log_connection;

	# Analyze HTTP input.
	$_ = <STDIN>;
	($method, $url, $proto) = split;
	logit("Request: $method $url $proto\n");
	if ($proto)
	{
		while (<STDIN>)
		{ 
			s#[\n\r]##mg;
			length || last; # empty line - end of header
			/^Content-Length: (\S*)/i && ($content_length = $1);
			/^Content-Type: (\S*)/i && ($content_type = $1);
			if (/^(\w+): (.*)/)
			{
				$one = uc($1);
				$ENV{"HTTP_$one"} = $2;
			}
		}
		my $lproto = lc($proto);
		if ($lproto eq "http/1.0")
		{
			# nothing.
		}
		elsif ($lproto eq "http/0.9")
		{
			$http_zero_nine = 1;
		}
		else
		{
			error(501, $proto, "");
		}
	}
	else
	{
		$proto = "HTTP/1.0";
	}

	my $lmethod = lc($method);
	if ($lmethod eq "get") 
	{
		# nothing.
	}
	elsif ($lmethod eq "head")
	{
		$no_body = 1;
	}
	elsif ($lmethod eq "post")
	{
		# nothing.
	}
	else
	{
		error(501, $method, "");
	}

	$url =~ s#/$#/index.html#;	# URL ending in / gets substituted
	$url =~ s#/~#/#;		# URL for username.

	# prevent directory go-back
	error(403, $url, "contains go-back") if (index($url, "..") > -1);

	# Check access control
	$allow = 0;
	foreach $k (keys %acl)
	{
		foreach $host (@host)
		{
			if ($host =~ /$k/i)
			{
				$acurl = $acl{$k};
				$deny = ($acurl =~ s/^!//);
				error(403, $url, "on deny list: $acurl $host") if ($deny && $url =~ /$acurl/);
				$allow = 1;
			}
		}
	}
	error(403, $url, "not on allow list") unless $allow;

	$url .= "/index.html" if -d "$htmldir$url";

	# if we've got a cgi-bin, first check if we are configured.  if so, and
	# we have a way to do cgi-bin, call it.  else, error.
	if ($url =~ m#^/cgi-bin/#)
	{
		unless ($cgidir)
		{
			error(501, $url, "cgi-bin has not been configured");
		}
		do_cgi_bin() if defined &do_cgi_bin;	# do_cgi_bin will exit or error.
		error(500, $url, "");
	}

	$file = "$htmldir$url";
	logit("opening file: $file");
	open (FILE, "<$file") || error(404, $url, "open failed: $!");
	$size = -s FILE;
	if ($url =~ /\.html$/)
	{
		$type = "text/html";
	}
	elsif ($url =~ /\.gif$/)
	{
		$type = "image/gif";
	}
	elsif ($url =~ /\.jpg$/)
	{
		$type = "image/jpg";
	}
	else
	{
		$type = "text/plain";
	}
	# output the file
	print <<TheEnd;
HTTP/1.0 200 OK
MIME-Version: 1.0
Server: $ENV{SERVER_SOFTWARE}
Content-Type: $type
Content-Length: $size

TheEnd
	unless ($no_body)
	{
		print while <FILE>;
	}
	close FILE;
	exit 0;
}

sub error
{
	# generate error response
	my ($errno, $url, $message) = @_;
	my ($errmsg) = "$errno $errors{$errno}";

	logit("$errmsg ($url, $message)");
	$var = <<TheEnd;
<HTML>
<HEAD><TITLE>$errmsg</TITLE></HEAD>
<BODY><H1>$errmsg</H1>
$verrors{$errno}: <PRE> $url </PRE>
<HR>
<ADDRESS><A HREF="ftp://ftp.eterna.com.au/bozo/bsf/attware/bozohttpd.tar.gz">
$ENV{'SERVER_SOFTWARE'}</A></ADDRESS>
</BODY>
</HTML>
TheEnd
	$size = length $var;
	print <<TheEnd;
HTTP/1.0 $errmsg
MIME-version: 1.0
Content-type: text/html
Context-length: $size;
Server: $ENV{SERVER_SOFTWARE}

$var
TheEnd
	exit;
}

sub logit
{
	my ($message) = @_;

	# use Syslog; when it's available.
	`logger -t '$0\[$$\]' -p $logfac.$logpri '$message'`;
}

sub death
{
	my ($message, $exit) = @_;
	logit($message);
	exit $exit;
}

sub killwarnings
{
	killwarnings($htmldir);
	$log_connection = $cgidir;
	$content_type = $content_length;
	$http_zero_nine = 1;
}
