#! /local/bin/perl -w
#
# $Id: bozohttpd.pl,v 2.10 1995/07/29 01:41:39 mrg Exp $
#
# bozo httpd -- based on tiny httpd.
#
# currently supported: HTTP 1.0 GET
# file types of .html and .gif
#
# bozoleft(p) 1995 the bozotic softwar foundation
#
# author: mrg@mame.mu.oz.au

require 5;

$0 =~ s#.*/##;

require '/etc/bozohttpd.cf';

death("no htmldir set in /etc/bozohttpd.cf", 1) unless $htmldir;

$logfac = "auth" unless $logfac;
$logpri = "info" unless $logpri;

use Socket 'AF_INET';

# Messages
%errors =
(
	"403" => "Forbidden",
	"404" => "Not Found",
	"500" => "Internal Error",
	"501" => "Not Implemented",
);

%verrors =
(
	"403" => "Your client is not allowed to request this item",
	"404" => "The requested item was not found on this server",
	"500" => "An error occurred while trying to retrieve item",
	"501" => "This server does not support the given request type",
);

$ENV{'SERVER_SOFTWARE'} = 'bozohttpd $Revision: 2.10 $';
$sockaddr = 'S n a4 x8';

($addr = getpeername(STDIN)) || death("getsockaddr: $!");
($inetaddr) = (unpack($sockaddr, $addr))[2];
($host, $aliases) = gethostbyaddr($inetaddr, AF_INET);
@inetaddr = unpack('C4', $inetaddr);
$inetaddr = join('.', @inetaddr);
@host = split(' ', "$host $aliases");
$host = $inetaddr unless $host;
$| = 1;

# Analyze HTTP input.
$_ = <STDIN>;
($method, $url, $proto) = split;
if ($proto)
{
	while (<STDIN>)
	{ 
		s/\n|\r//g; # kill CR and NL chars
		/^Content-Length: (\S*)/i && ($content_length = $1);
		/^Content-Type: (\S*)/i && ($content_type = $1);
		length || last; # empty line - end of header
	}
}
else
{
	$proto = "HTTP/1.0";
}
error(501, $method, "") unless $method =~ /^GET$/i;
error(501, $proto, "") unless $proto eq "HTTP/1.0";

$url =~ s#/$#/index.html#;	# URL ending in / gets substituted
$url =~ s#/~#/#;		# URL for username.
logit("Request: $method $url\n");
# prevent directory go-back
error(403, $url, "contains go-back") if (index($url, "..") > -1);

# Check access control
$allow = 0;
foreach $k (keys %acl)
{
	foreach $host (@host)
	{
		if ($host =~ /$k/i)
		{
			$acurl = $acl{$k};
			$deny = ($acurl =~ s/^!//);
			error(403, $url, "on deny list: $acurl $host") if ($deny && $url =~ /$acurl/);
			$allow = 1;
		}
	}
}
error(403, $url, "not on allow list") unless $allow;

$url .= "/index.html" if (-d "$htmldir$url");

if ($url =~ m#^/cgi-bin/#)
{
	unless ($cgidir)
	{
		error(501, $url, "cgi-bin has not been configured");
	}
	# TODO: PATH_INFO, PATH_TRANSLATED, QUERY_STRING
	$ENV{'SERVER_NAME'} =		$server_name if $server_name;
	$ENV{'CONTENT_TYPE'} =		$content_type if $content_type;
	$ENV{'CONTENT_LENGTH'} =	$content_length if $content_length;
	$ENV{'GATEWAY_INTERFACE'} =	"CGI/1.0";
	$ENV{'SERVER_PORT'} =		80;
	$ENV{'SERVER_PROTO'} =		$proto;
	$ENV{'REQUEST_METHOD'} =	$method;
	$ENV{'REMOTE_ADDR'} =		$inetaddr;

	# generate command line
	$file = $url;
	$file =~ s#^/cgi-bin#$cgidir#o;
	$file =~ s/\?.*$//;
	$srch = $url;
	$srch =~ s/^[^\?]*\??//;
	$srch = "" if (length($srch) > 250); # safe side
	return error(404, $url, "$file -x failed: $!") unless -x $file;
	logit("executing: $url ($file $srch)");

	($ENV{'SCRIPT_NAME'} = $file) =~ s#./*##;

	# Parsing of sent-back headers is not implemented. 
	# Script effectively talks back to client.
	print "HTTP/1.0 200 OK\nMIME-Version: 1.0\n";
	open(CGI, "|$file $srch") || return error(500, $url, "pipe: $!");
	select(CGI);
	$| = 1;
	$content_length = 65536 unless $content_length;	# hmm??
	while (1)
	{
		# don't read past content-length - client doesn't give EOF
		$len = length;
		s/\r//g;
		# print LOG;
		print CGI;
		last if ($content_length -= $len) <= 0;
		$_ = <STDIN>;
	}
	close CGI;
	exit 0;
}
else
{
	$file = "$htmldir$url";
	logit("opening file: $file");
	open (FILE, "<$file") || error(404, $url, "open failed: $!");
	# output the file
	print "HTTP/1.0 200 OK\nMIME-Version: 1.0\nContent-Type: ";
	if ($url =~ /\.html$/)
	{
		print "text/html\n\n";
	}
	elsif ($url =~ /\.gif$/)
	{
		print "image/gif\n\n";
	}
	else
	{
		print "text/plain\n\n";
	}
	print while <FILE>;
	close FILE;
	exit 0;
}

sub error
{
	# generate error response
	my ($errno, $url, $message) = @_;
	my ($errmsg) = "$errno $errors{$errno}";

	logit("$errmsg ($url, $message)");
	print <<TheEnd;
HTTP/1.0 $errmsg
MIME-version: 1.0
Content-type: text/html

<HTML>
<HEAD><TITLE>$errmsg</TITLE></HEAD>
<BODY><H1>$errmsg</H1>
$verrors{$errno}: <PRE> $url </PRE>
<HR>
<ADDRESS><A HREF="ftp://ftp.eterna.com.au/bozo/bsf/attware/bozohttpd.tar.gz">
$ENV{'SERVER_SOFTWARE'}</A></ADDRESS>
</BODY>
</HTML>
TheEnd
	exit;
}

sub logit
{
	my ($message) = @_;

	`logger -t '$0\[$$\]' -p $logfac.$logpri '$message'`;
}

sub death
{
	my ($message, $exit) = @_;
	logit($message);
	exit $exit;
}

sub killwarnings
{
	killwarnings($htmldir);
}
